package org.asciidoctor.ast.impl;

import org.asciidoctor.ast.Cursor;
import org.asciidoctor.ast.StructuralNode;
import org.asciidoctor.internal.RubyBlockListDecorator;
import org.asciidoctor.internal.RubyHashUtil;
import org.jruby.RubyArray;
import org.jruby.runtime.builtin.IRubyObject;

import java.util.List;
import java.util.Map;

public class StructuralNodeImpl extends ContentNodeImpl implements StructuralNode {

    private static final String BLOCK_CLASS = "Block";
    private static final String SECTION_CLASS = "Section";
    
    public StructuralNodeImpl(IRubyObject blockDelegate) {
        super(blockDelegate);
    }

    @Override
    public String title() {
        return getTitle();
    }

    @Override
    public String getTitle() {
        return getString("title");
    }

    @Override
    public void setTitle(String title) {
        setString("title", title);
    }

    @Override
    public String style() {
        return getStyle();
    }

    @Override
    public String getStyle() {
        return getString("style");
    }

    @Override
    public List<StructuralNode> blocks() {
        return getBlocks();
    }

    @Override
    public List<StructuralNode> getBlocks() {
        RubyArray rubyBlocks = (RubyArray) getRubyProperty("blocks");
        return new RubyBlockListDecorator<StructuralNode>(rubyBlocks);
    }

    @Override
    public void append(StructuralNode block) {

        getRubyObject().callMethod(runtime.getCurrentContext(), "<<", ((StructuralNodeImpl) block).getRubyObject());
    }

    @Override
    public Object content() {
        return getContent();
    }

    @Override
    public Object getContent() {
        return getProperty("content");
    }

    @Override
    public String convert() {
        return getString("convert");
    }

    @Override
    public int getLevel() {
        return getInt("level");
    }

    @Override
    public Cursor getSourceLocation() {
        IRubyObject object = getRubyProperty("source_location");
        if (object == null || object.isNil()) {
            return null;
        }
        return new CursorImpl(object);
    }

    @Override
    public List<StructuralNode> findBy(Map<Object, Object> selector) {

        RubyArray rubyBlocks = (RubyArray) getRubyProperty("find_by", RubyHashUtil.convertMapToRubyHashWithSymbolsIfNecessary(runtime,
                selector));
        return new RubyBlockListDecorator(rubyBlocks);
    }

}
